;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Mongoose's .emacs / init.el
;;------------------------------------------------
;;
;; Key map
;;------------------------------------------------
;; F1 - Man lookup on symbol/name
;; F5 - Compile
;; F6 - Debug
;; F8 - Next compile error
;; F9 - Recompile ( quick, any buffer )
;;
;; C-mouse1 - Insert C++ method defn based on template
;; C-mouse2 - Insert C++ line comment based on template
;;
;; Scroll mouse support 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Load my ~/scripts path and any scripts
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(setq load-path
      (append (list (expand-file-name "~/scripts")) load-path))

;; Load C++ template elisp functions
(autoload 'cpp-class-template  "cpp" "Mongoose's C++ template generator." t)
(autoload 'cpp-method-template "cpp" "Mongoose's C++ template generator." t)
(autoload 'cpp-line-comment    "cpp" "Mongoose's C++ template generator." t)
(autoload 'java-class-template "cpp" "Mongoose's C++ template generator." t)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Custom key commands
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(global-set-key [(control button1)] 'cpp-method-template) ; C-mouse1 
(global-set-key [(control button2)] 'cpp-line-comment) ; C-mouse2
(global-set-key [mouse-4] 'scroll-down)
(global-set-key [mouse-5] 'scroll-up)

; Man (help) on symbol with F1 key
(global-set-key [(f1)] (lambda () (interactive) (manual-entry (current-word))))

; Completion support
;(global-set-key (quote [f2]) 'dabbrev-expand)
;(global-set-key (quote [f3]) 'dabbrev-completion)

; Compile with F5 key
(global-set-key (quote [f5]) 'compile)

; Reompile with F4 key
(global-set-key (quote [f4]) 'recompile)

; Debug with F6 key
(global-set-key (quote [f6]) 'gdb)

; Thanks Neo for this quick-compile snippet
(defun quick-compile ()
  "Rerun the last compilation command"
  (interactive)
  (let ((buf (get-buffer "*compilation*")))
    (if buf
        (if (equal buf (current-buffer))
            nil
	  (switch-to-buffer-other-window buf))
      (error "No previous compilation"))
    (compile compile-command)))

; Next error on F8 Key
(global-set-key [f8] 'next-error)

; Quick compile on F9 key
(global-set-key [f9] 'quick-compile)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Adjust for my programming style
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(require 'cc-mode)
(c-set-offset 'substatement-open 0)
(c-set-offset 'statement-cont 0)
(setq-default indent-tabs-mode t)
(setq tab-width 3)
;(add-hook 'c-mode-common-hook
;	  (lambda () (c-toggle-hungry-state 1)))
;
;(defconst my-c-style 
;  '((c-tab-always-indent        . t) 
;    (c-comment-only-line-offset . 4) 
;    (c-basic-offset             . 3) 
;    (c-hanging-braces-alist     . ((substatement-open after) 
;				   (brace-list-open))) 
;    (c-hanging-colons-alist     . ((member-init-intro before)  
;				   (inher-intro)  
;				   (case-label after)  
;				   (label after)  
;				   (access-label after)))  
;    (c-cleanup-list             . (scope-operator  
;				   empty-defun-braces  
;				   defun-close-semi))  
;    (c-offsets-alist            . ((arglist-close . c-lineup-arglist) 
;				   (substatement-open . 0) 
;				   (case-label        . 4) 
;				   (block-open        . 0)
;				   (inamespace        . 0)
;				   (knr-argdecl-intro . -)))

;; Custom look and feel
;(setq options-save-faces t)
;(set-face-background-pixmap 'default "~/.xemacs/gnome3.xpm")
;(set-face-background-pixmap 'buffers-tab "~/.xemacs/gnome3.xpm")
(set-face-background 'default      "gray60")
(set-face-background 'modeline     "gray68")
(set-face-background 'isearch      "yellow")
(set-face-background 'zmacs-region "white")
(set-face-foreground 'zmacs-region "black")


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Semantic
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(setq semantic-load-turn-everything-on t)
(load-file "/usr/share/xemacs21/packages/lisp/semantic/semantic-load.elc")


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Speedbar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(autoload 'speedbar-frame-mode "speedbar" "Popup a speedbar frame" t)
;(autoload 'speedbar-get-focus "speedbar" "Jump to speedbar frame" t)

(require 'speedbar)
; Add speedbar to tools menu
(add-menu-button '("Tools")
		 ["Speedbar" speedbar-frame-mode
		 :style toggle
		 :selected (and (boundp 'speedbar-frame)
				(frame-live-p speedbar-frame)
				(frame-visible-p speedbar-frame))]
				"--")

; Adjust dim, pos of speedbar
(setq speedbar-frame-plist (append '(top 64 left 16 height 41 width 25)
                                   speedbar-frame-plist))

;(speedbar-show-unknown-files t)
;(speedbar-frame-mode) ; Auto load at start ( moved to custom.el )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; OpenGL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(autoload 'OpenGL-minor-mode "OpenGL" "OpenGL editing utilities." t)
;(add-hook 'OpenGL-minor-mode-hook 'OpenGL-setup-keys)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Recently used menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(require 'recentf)
;(recentf-mode 1)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; OO Browser
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(setq load-path (append
;		 '("~/scripts/oo-browser/"
;		   "~/scripts/oo-browser/hypb/")
;		 load-path))

;(load "br-start")

;(global-set-key "\C-c\C-o" 'oo-browser)

