################################################################
# Mongoose <stu7440@westga.edu>
################################################################
# + Cleaner clean
# + Add new include tree
# + Add new flags
# + Install
################################################################

NAME=#Project
MAJOR_VERSION=0
MINOR_VERSION=0
MICRO_VERSION=0
BUILD_ID=#YEAR#MONTH#DAY
VERSION=$(MAJOR_VERSION).$(MINOR_VERSION).$(MICRO_VERSION)

################################################################

BUILD_SELECT=debug
BUILD_DEBUG_DIR=bin/prof
BUILD_DEBUG_DIR=bin/debug
BUILD_RELEASE_DIR=bin/release
BUILD_TEST_DIR=bin/test
BUILD_INSTALL_DIR=bin/$(BUILD_SELECT)
INSTALL_DIR=/usr/local/bin
TREE_DIR=#Project

################################################################

CC=gcc
JAVA=javac
FLAGS=

################################################################

DEBUG_DEFS=-DDEBUG -DEXPERIMENTAL

BASE_CFLAGS=$(FLAGS) -Wall \
		-DVERSION=\"\\\"$(VERSION)\\\"\" \
		-DBUILD_ID=\"\\\"$(BUILD_ID)\\\"\" \
		-DPROGRAM_NAME=\"\\\"$(NAME)-$(VERSION)-$(BUILD_ID)\\\"\"

LD_FLAGS=-static -fPIC


RELEASE_CFLAGS=$(BASE_CFLAGS) \
	-ffast-math -funroll-loops \
	-fomit-frame-pointer -fexpensive-optimizations -O2

DEBUG_CFLAGS=$(BASE_CFLAGS) -g $(DEBUG_DEFS)

DO_CC=$(CC) $(CFLAGS) -o $@ -c $<

################################################################


TARGETS=$(BUILD_DIR)/$(NAME)

auto: $(BUILD_SELECT)

################################################################

release:
	@-mkdir -p $(BUILD_RELEASE_DIR)
	$(MAKE) targets BUILD_DIR=$(BUILD_RELEASE_DIR) \
	CFLAGS="$(RELEASE_CFLAGS)" \
	LD_FLAGS="$(LD_FLAGS)"

debug:
	@-mkdir -p $(BUILD_DEBUG_DIR)
	$(MAKE) targets BUILD_DIR=$(BUILD_DEBUG_DIR) \
	CFLAGS="$(DEBUG_CFLAGS)" \
	LD_FLAGS="$(LD_FLAGS)"

prof: 
	@-mkdir -p $(BUILD_PROF_DIR)
	$(MAKE) targets BUILD_DIR=$(BUILD_PROF_DIR) \
	CFLAGS="$(DEBUG_CFLAGS) -pg" \
	LD_FLAGS="$(LD_FLAGS) -pg"

########################################################################

all: debug release prof

########################################################################

targets: $(TARGETS)

OBJS = \
	$(BUILD_DIR)/.o

$(BUILD_DIR)/$(NAME) : $(OBJS) 
	$(CC) $(CFLAGS) -o $@ $(OBJS) $(LD_FLAGS)

$(BUILD_DIR)/.o :             src/.c
	$(DO_CC)



################################################################

clean: 
	@-echo -n "Cleaning emacs files                        "
	@-rm -f `find . -name "*~" -print`
	@-echo [done]

	@-echo -n "Cleaning profiling output                   "
	@-rm -f gmon.out
	@-echo [done]

	@-echo -n "Cleaning TEST builds                        "
	@-rm -f $(BUILD_TEST_DIR)/*.test
	@-echo [done]

	@-echo -n "Cleaning DEBUG builds                       "
	@-rm -f $(BUILD_DEBUG_DIR)/*.o
	@-rm -f $(BUILD_DEBUG_DIR)/$(NAME)
	@-echo [done]

	@-echo -n "Cleaning PROF builds                        "
	@-rm -f $(BUILD_PROF_DIR)/*.o
	@-rm -f $(BUILD_PROF_DIR)/$(NAME)
	@-echo [done]

	@-echo -n "Cleaning RELEASE builds                     "
	@-rm -f $(BUILD_RELEASE_DIR)/*.o
	@-rm -f $(BUILD_RELEASE_DIR)/$(NAME)
	@-echo [done]


################################################################

install:
	@-echo -n "Installing                                  "
	@-cp $(BUILD_INSTALL_DIR)/$(NAME) $(INSTALL_DIR)
	@-echo [done]


################################################################

uninstall:
	@-echo -n "Uninstalling                                "
	@-rm -f $(INSTALL_DIR)/$(NAME) 
	@-echo [done]

################################################################

tarball:
	@-$(MAKE) clean
	@-cd .. && tar zcvf $(NAME)-$(VERSION)-$(BUILD_ID).tar.gz \
		$(TREE_DIR) \
		--exclude bin --exclude *~ --exclude cvs  \
		--exclude CVS --exclude *.notar --exclude spec \
		--exclude *.log --exclude contrib \
		--totals

backup:
	@-$(MAKE) clean
	@-cd .. && tar Icvf $(NAME)-$(VERSION)-$(BUILD_ID).tar.bz2 $(TREE_DIR)



################################################################

test:
	@-echo "No test build rule made"

